@@ -95,6 +95,12 @@ impl ScanError {
         }
     }
 
+    /// Create a new error from a location and an error string.
+    #[must_use]
+    pub fn new_string(loc: Marker, info: String) -> ScanError {
+        ScanError { mark: loc, info }
+    }
+
     /// Return the marker pointing to the error in the source.
     #[must_use]
     pub fn marker(&self) -> &Marker {
@@ -135,7 +141,7 @@ impl fmt::Display for ScanError {
 /// The contents of a scanner token.
 #[derive(Clone, PartialEq, Debug, Eq)]
 pub enum TokenType {
-    /// The start of the stream. Sent first, before even [`DocumentStart`].
+    /// The start of the stream. Sent first, before even [`TokenType::DocumentStart`].
     StreamStart(TEncoding),
     /// The end of the stream, EOF.
     StreamEnd,
@@ -312,6 +318,7 @@ struct Indent {
 /// consume/push a character. As of now, almost all lookaheads are 4 characters maximum, except:
 ///   - Escape sequences parsing: some escape codes are 8 characters
 ///   - Scanning indent in scalars: this looks ahead `indent + 2` characters
+///
 /// This constant must be set to at least 8. When scanning indent in scalars, the lookahead is done
 /// in a single call if and only if the indent is `BUFFER_LEN - 2` or less. If the indent is higher
 /// than that, the code will fall back to a loop of lookaheads.
@@ -324,8 +331,8 @@ const BUFFER_LEN: usize = 16;
 /// some of the constructs. It has understanding of indentation and whitespace and is able to
 /// generate error messages for some invalid YAML constructs.
 ///
-/// It is however not a full parser and needs [`parser::Parser`] to fully detect invalid YAML
-/// documents.
+/// It is however not a full parser and needs [`crate::parser::Parser`] to fully detect invalid
+/// YAML documents.
 #[derive(Debug)]
 #[allow(clippy::struct_excessive_bools)]
 pub struct Scanner<T> {
@@ -505,7 +512,7 @@ impl<T: Iterator<Item = char>> Scanner<T> {
 
     /// Consume a linebreak (either CR, LF or CRLF), if any. Do nothing if there's none.
     #[inline]
-    fn skip_line(&mut self) {
+    fn skip_linebreak(&mut self) {
         if self.buffer[0] == '\r' && self.buffer[1] == '\n' {
             // While technically not a blank, this does not matter as `self.leading_whitespace`
             // will be reset by `skip_nl`.
@@ -842,7 +849,7 @@ impl<T: Iterator<Item = char>> Scanner<T> {
                 '\t' | ' ' => self.skip_blank(),
                 '\n' | '\r' => {
                     self.lookahead(2);
-                    self.skip_line();
+                    self.skip_linebreak();
                     if self.flow_level == 0 {
                         self.allow_simple_key();
                     }
@@ -873,7 +880,7 @@ impl<T: Iterator<Item = char>> Scanner<T> {
                 }
                 '\n' | '\r' => {
                     self.lookahead(2);
-                    self.skip_line();
+                    self.skip_linebreak();
                     if self.flow_level == 0 {
                         self.allow_simple_key();
                     }
@@ -970,8 +977,6 @@ impl<T: Iterator<Item = char>> Scanner<T> {
         self.disallow_simple_key();
 
         let tok = self.scan_directive()?;
-        self.skip_ws_to_eol(SkipTabs::Yes)?;
-
         self.tokens.push_back(tok);
 
         Ok(())
@@ -1003,20 +1008,16 @@ impl<T: Iterator<Item = char>> Scanner<T> {
 
         self.skip_ws_to_eol(SkipTabs::Yes)?;
 
-        if !is_breakz(self.ch()) {
-            return Err(ScanError::new(
+        if is_breakz(self.ch()) {
+            self.lookahead(2);
+            self.skip_linebreak();
+            Ok(tok)
+        } else {
+            Err(ScanError::new(
                 start_mark,
                 "while scanning a directive, did not find expected comment or line break",
-            ));
+            ))
         }
-
-        // Eat a line break
-        if is_break(self.ch()) {
-            self.lookahead(2);
-            self.skip_line();
-        }
-
-        Ok(tok)
     }
 
     fn scan_version_directive_value(&mut self, mark: &Marker) -> Result<Token, ScanError> {
@@ -1145,7 +1146,7 @@ impl<T: Iterator<Item = char>> Scanner<T> {
                     self.scan_tag_shorthand_suffix(false, is_secondary_handle, "", &start_mark)?;
             } else {
                 suffix = self.scan_tag_shorthand_suffix(false, false, &handle, &start_mark)?;
-                handle = "!".to_owned();
+                "!".clone_into(&mut handle);
                 // A special case: the '!' tag.  Set the handle to '' and the
                 // suffix to '!'.
                 if suffix.is_empty() {
@@ -1778,7 +1779,7 @@ impl<T: Iterator<Item = char>> Scanner<T> {
             self.mark.index += line_buffer.len();
 
             // We can now append our bytes to our `string`.
-            string.reserve(line_buffer.as_bytes().len());
+            string.reserve(line_buffer.len());
             string.push_str(line_buffer);
             // This clears the _contents_ without touching the _capacity_.
             line_buffer.clear();
@@ -1800,7 +1801,10 @@ impl<T: Iterator<Item = char>> Scanner<T> {
                     while !self.buffer.is_empty() && self.mark.col < indent && self.ch() == ' ' {
                         self.skip_blank();
                     }
-                    if !(!self.buffer.is_empty() && self.mark.col < indent && self.ch() == ' ') {
+                    // If we reached our indent, we can break. We must also break if we have
+                    // reached content or EOF; that is, the buffer is not empty and the next
+                    // character is not a space.
+                    if self.mark.col == indent || (!self.buffer.is_empty() && self.ch() != ' ') {
                         break;
                     }
                 }
@@ -2042,7 +2046,7 @@ impl<T: Iterator<Item = char>> Scanner<T> {
                 '\\' if !single && is_break(self.buffer[1]) => {
                     self.lookahead(3);
                     self.skip_non_blank();
-                    self.skip_line();
+                    self.skip_linebreak();
                     *leading_blanks = true;
                     break;
                 }
@@ -2455,7 +2459,12 @@ impl<T: Iterator<Item = char>> Scanner<T> {
     /// An indentation is not added if we are inside a flow level or if the last indent is already
     /// a non-block indent.
     fn roll_one_col_indent(&mut self) {
-        if self.flow_level == 0 && self.indents.last().map_or(false, |x| x.needs_block_end) {
+        if self.flow_level == 0
+            && self
+                .indents
+                .last()
+                .map_or(false, |indent| indent.needs_block_end)
+        {
             self.indents.push(Indent {
                 indent: self.indent,
                 needs_block_end: false,
